<?php
/**
 * @version		$Id: jailserver_manager.class.php,v 1.11 2011-01-02 21:40:37 juanca Exp $
 * @package		vpl. vpl jailservers manager class definition
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

/**
 * vpl_jailserver_manager is a utility class to manage
 * the jail servers. get_Server if the key function
 *
 */
class vpl_jailserver_manager{
	const recheck=300; //300 sec = 5 min ¿optional setable?
	const table='vpl_jailservers';

	/**
	 * Check if the server is tagged as down
	 * @param url $server
	 * @return boolean
	 */
	static private function is_checkable($server){
		$info= get_record('vpl_jailservers','server',$server);
		if($info != null){
			if($info->lastfail+self::recheck > time()){
				return false;
			}
		}
		return true;
	}

	/**
	 * Tag the server as down
	 * @param URL $server
	 * @param string $strerror
	 * @return void
	 */
	static private function server_fail($server,$strerror){
		if($strerror==null){
			$strerror='';
		}
		$info= get_record(self::table,'server',$server);
		if($info != null){
			$info->lastfail=time();
			$info->laststrerror=$strerror;
			$info->nfails++;
			update_record(self::table,$info);
		}else{
			$info = new stdClass();
			$info->server = $server;
			$info->lastfail=time();
			$info->laststrerror=$strerror;
			$info->nfails=1;
			insert_record(self::table,$info);
		}
	}

	/**
	 * Return a valid server to be used, May tag some servers as faulty
	 * @param int $maxmemory requiered
	 * @param string $localserverlisttext='' List of local server in text
	 * @param string $feedback info about jail servers response
	 * @return URL
	 */
	static function get_server($maxmemory,$localserverlisttext='', &$feedback=null){
		if(!function_exists('xmlrpc_encode_request')){
			vpl_js_alert('Error: PHP xmlrpc requiered');
			print_error('Error: PHP xmlrpc requiered');
			die;
		}
		global $CFG;
		$data = new stdClass();
		$data->maxmemory=$maxmemory;
		$add_global_servers = true;
		$localserverlist = array();
		$globalserverlist = array();
		if($localserverlisttext>''){
			//Clean local server list and search for 'end_of_jails'
			$nl = vpl_detect_newline($localserverlisttext);
			$tempserverlist = explode($nl,$localserverlisttext);
			foreach ($tempserverlist as $server) {
				$server = trim($server);
				if($server>'' && $server[0]!='#'){
					if(strtolower($server) == 'end_of_jails'){
						$add_global_servers = false;
						break;
					}else{
						$localserverlist[] = $server;
					}
				}
			}
		}
		if($add_global_servers){
			$nl = vpl_detect_newline($CFG->vpl_jail_servers);
			$globalserverlist = explode($nl,$CFG->vpl_jail_servers);
		}
		//Merge local and global server list
		$serverlist = array_merge($globalserverlist, $localserverlist);
		$requestReady = xmlrpc_encode_request('status',$data);
		shuffle($serverlist);
		$http=null;
		$feedback = '';
		foreach (array(0=>false,1=>true) as $checkall){
			foreach ($serverlist as $server) {
				$server = trim($server);
				if($server>'' && $server[0]!='#' &&
				($checkall || vpl_jailserver_manager::is_checkable($server))){
					$http = new vpl_HTTP_request($requestReady);
					if($http->try_server($server,4)){
						$raw_response=$http->get_response();
						$response = xmlrpc_decode($raw_response);
						if(is_array($response) && xmlrpc_is_fault ($response)){
							$feedback .= $server .' xmlrpc is fault: '.$response["faultString"]."\n";
							p(print_r($response));
						}else{
							$status=$response['status'];
							if($status == 'ready'){
								return $server;
							}
							else if($status!='busy'){
								//TODO need actions?
								//vpl_jailserver_manager::server_fail($server,'Alive but not ready :'.$raw_response);
								$feedback .= $server .' busy or without resources to acept the request'."\n";
								continue;
							}
						}
					}
					$feedback .= $server .' request failed: '.$http->get_error()."\n";
					vpl_jailserver_manager::server_fail($server,$http->get_error());
				}
			}
		}
		return false;
	}

	/**
	 * Clear servers table and check for every one again
	 * @return string server list with info
	 */
	static function reset_servers(){
		global $CFG;
		delete_records('vpl_jailservers');
		$nl = vpl_detect_newline($CFG->vpl_jail_servers);
		$serverlist = explode($nl,$CFG->vpl_jail_servers);
		$requestReady = xmlrpc_encode_request('status',null);
		$result='';
		$http=null;
		foreach ($serverlist as $server) {
			$server = trim($server);
			if($server>'' && vpl_jailserver_manager::is_checkable($server)){
				$http = new vpl_HTTP_request($requestReady);
				if($http->try_server($server,4)){
					$response = xmlrpc_decode($http->get_response());
					//TODO use new format
					if($response == 'yes'){
						$result .=$server+" (OK)\n";
					}
					else{
						$result .=$server+' (Alive but not ready: '.$http->get_response().")\n";
					}
				}else{
					vpl_jailserver_manager::server_fail($server,$http->get_error());
					$result .=$server+' (Not ready:'.$http->get_error().")\n";
				}
			}
		}
		return $result;
	}
}

?>