<?php
/**
 * @version		$Id: lib.php,v 1.69 2010-12-21 17:44:49 juanca Exp $
 * @package		vpl. functions to coordinate with moodle
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

require_once dirname(__FILE__).'/locallib.php';

/**
 * Create grade item for vpl
 *
 * @param object $assignment object with extra cmidnumber
 * @param mixed optional array/object of grade(s); 'reset' means reset grades in gradebook
 * @return int 0 if ok, error code otherwise
 */
function vpl_update_grade_item($instance,$cm=null) {
	$itemdetails = array('itemname' => $instance->name);
	if($cm){
		$itemdetails['idnumber']= $cm->id;
	}
	if ($instance->grade > 0) {
		$itemdetails['gradetype'] = GRADE_TYPE_VALUE;
		$itemdetails['grademax']  = $instance->grade;
		$itemdetails['grademin']  = 0;
	} else if ($instance->grade < 0) {
		$itemdetails['gradetype'] = GRADE_TYPE_SCALE;
		$itemdetails['scaleid']   = -$instance->grade;
	} else {
		$itemdetails = array('deleted'=>1);
	}
	$itemdetails['hidden'] = ($instance->visiblegrade)?0:1;
	grade_update('mod/vpl', $instance->course, 'mod', VPL,
					$instance->id, 0, NULL, $itemdetails);
}

/**
 * Delete grade_item from a vpl instance+id
 *
 * @param $instance vpl instance
 **/
function vpl_delete_grade_item($instance) {
	global $CFG;
	require_once $CFG->libdir.'/gradelib.php';
	$itemdetails = array('deleted'=>1);
	grade_update('mod/vpl', $instance->course, 'mod', VPL,$instance->id, 0, NULL, $itemdetails);
}


/**
 * Create an event object from a vpl instance+id
 *
 * @param $instance vpl instance
 * @param $id vpl instance id
 * @return event object
 **/
function vpl_create_event($instance,$id){
	$event = new stdClass();
	$event->name        = $instance->name;
	$event->description = $instance->shortdescription;
	$event->courseid    = $instance->course;
	$event->modulename  = VPL;
	$event->instance    = $id;
	$event->eventtype   = 'due';
	$event->timestart   = $instance->duedate;
	return $event;
}

/**
 * Add a new vpl instance and return the id
 *
 * @param object from the form in mod_form.html
 * @return int id of the new vpl
 **/
function vpl_add_instance($instance) {
	$id = insert_record(VPL, $instance);
	//Add event
	if ($instance->duedate) {
		add_event(vpl_create_event($instance,$id));
	}
	//Add grade to grade book
	$instance->id=$id;
	vpl_update_grade_item($instance);
	return $id;
}

/**
 * Update a vpl instance
 *
 * @param object from the form in mod.html
 * @return boolean OK
 *
 **/
function vpl_update_instance($instance) {
	$instance->id = $instance->instance;
	//Update event
	if ($instance->duedate) {
		$event = vpl_create_event($instance,$instance->id);
		if ($eventid = get_field('event', 'id', 'modulename', VPL, 'instance', $instance->id)) {
			$event->id = $eventid;
			update_event($event);
		} else {
			add_event($event);
		}
	} else {
		delete_records('event', 'modulename', VPL, 'instance', $instance->id);
	}
	$cm = get_coursemodule_from_instance(VPL, $instance->id, $instance->course);
	vpl_update_grade_item($instance,$cm);
	return update_record(VPL, $instance);
}

/**
 * Delete an instance by id
 *
 * @param int $id Id instance
 * @return boolean OK
 **/
function vpl_delete_instance($id) {
	$vpl = new mod_vpl(false,$id);
	$res = $vpl->delete_all();
	//Locate related VPLs and reset its basedon $id to 0
	$related = get_records_select(VPL,"basedon = $id",'id','id,basedon');
	if($related !== false){
		foreach($related as $other){
			$other->basedon=0;
			update_record(VPL,$other);
		}
	}
	return $res;
}

/**
 * Return an object with short information about what a
 * user has done with a given particular instance of this module
 * $return->time = the time they did it
 * $return->info = a short text description
 * @param $course
 * @param $user
 * @param $mod
 * @param $instance
 * @return the object info
 **/
function vpl_user_outline($course, $user, $mod, $instance) {
	//Search submisions for $user $instance
	$vpl = new mod_vpl(null,$instance->id);
	$subinstance = $vpl->last_user_submission($user->id);
	if(!$subinstance) {
		$return = null;
	}
	else{
		require_once('vpl_submission.class.php');
		$return = new stdClass;
		$submission = new mod_vpl_submission($vpl,$subinstance);
		$return->time= $subinstance->datesubmitted;
		$subs=$vpl->user_submissions($user->id);
		if(count($subs)>1){
			$info = get_string('submissions',VPL,count($subs));
		}else{
			$info = get_string('submission',VPL,count($subs));
		}
		if($subinstance->dategraded){
			$info .='<br />'.get_string('grade').': '.$submission->print_grade_core();
		}
		$url=vpl_mod_href('forms/submissionview.php','id',$vpl->get_course_module()->id,'userid',$user->id);
		$return->info = '<a href="'.$url.'">'.$info.'</a>';
	}
	return $return;
}

/**
 * Print a detailed report of what a user has done
 * with a given particular instance of this module
 * @param $course
 * @param $user
 * @param $mod
 * @param $instance
 * @return boolean OK
 **/
function vpl_user_complete($course, $user, $mod, $vpl) {
	require_once('vpl_submission.class.php');
	//TODO Print a detailed report of what a user has done with a given particular instance
	//Search submisions for $user $instance
	$vpl = new mod_vpl(null,$vpl->id);
	$sub = $vpl->last_user_submission($user->id);
	if(!$sub) {
		$return = null;
	}
	else{
		$submission = new mod_vpl_submission($vpl,$sub);
		$submission->print_info(true);
		$submission->print_grade(true);
	}
	return true;
}

/**
 * Returns all VPL assignments since a given time in specified forum.
 */
function vpl_get_recent_mod_activity(&$activities, &$index, $timestart, $courseid, $cmid, $userid=0, $groupid=0)  {
    //TODO group control
	global $CFG;
    global $USER;
	$grader = false;
	$vpl = new mod_vpl($cmid);
    $modinfo =& get_fast_modinfo($vpl->get_course());
    $cm = $modinfo->cms[$cmid];
	if ($vpl->is_visible() && $vpl->has_capability(VPL_VIEW_CAPABILITY)) {
		$vplid = $vpl->get_instance()->id;
		$grader = $vpl->has_capability(VPL_GRADE_CAPABILITY);
	}else{
		return;
	}
    $select = '(vpl = '.$vplid.')';
    $select .= ' and ((datesubmitted >= '.$timestart.') or (dategraded >= '.$timestart.'))';
    if(!$grader){ //Own activity
    	$select = '(userid = '.$USER->id.') and '.$select;
    }
    $subs = get_records_select(VPL_SUBMISSIONS,$select,'datesubmitted DESC');
    if (!$subs){ //No submission available
        return;
    }
    $aname = format_string($vpl->get_instance()->name,true);
    foreach ($subs as $sub) { //Show recent activity
        $activity = new stdClass();

        $activity->type         = 'vpl';
        $activity->cmid         = $cm->id;
        $activity->name         = $aname;
        $activity->sectionnum   = $cm->sectionnum;
        $activity->timestamp    = $sub->datesubmitted;

        if ($grader) {
            $activity->grade = $sub->grade;
        }
		$activity->user = get_record('user','id',$sub->userid);
        $activities[$index++] = $activity;
    	
    }
    return true;
}

/**
 *
 */
function vpl_print_recent_mod_activity($activity, $courseid, $detail, $modnames, $viewfullnames) {
    //TODO improve
    global $CFG;
    echo '<table border="0" cellpadding="3" cellspacing="0" class="forum-recent">';
    echo "<tr><td class=\"userpicture\" valign=\"top\">";
    print_user_picture($activity->user, $courseid);
    echo '</td><td>';

    echo '<div class="title">';
/*    if ($detail) {
        $aname = s($activity->name);
        echo "<img src=\"$CFG->modpixpath/$activity->type/icon.gif\" ".
             "class=\"icon\" alt=\"{$aname}\" />";
    }
    echo '<a href="'.vpl_mod_href('forms/view.$CFG->wwwroot/mod/forum/discuss.php?d={$activity->content->discussion}"
         ."#p{$activity->content->id}\">{$activity->content->subject}</a>";
    echo '</div>';
*/
    echo '<div class="user">';
    $fullname = fullname($activity->user, $viewfullnames);
    echo "<a href=\"$CFG->wwwroot/user/view.php?id={$activity->user->id}&amp;course=$courseid\">"
         ."{$fullname}</a> - ";
    $link = vpl_mod_href('forms/submissionview.php','id',$activity->cmid,'userid',$activity->user->id,'inpopup',1);
    echo '<a href="'.$link.'">'.userdate($activity->timestamp).'</a>';
    echo '</div>';
    echo "</td></tr></table>";

    return;
}



/**
 * Print activity of a course since a time
 *
 * @uses $CFG
 * @param $course instance object
 * @param $isteacher (not used here)
 * @param $timestart activity from
 * @return boolean (true==something printed)
 **/
function vpl_print_recent_activity($course, $isteacher, $timestart) {
    //TODO group control
	global $CFG;
    global $USER;
    if (!$cms = get_coursemodules_in_course(VPL, $course->id)) {
		return false;
	}
	$vpls = array();
	$grader = false;
	//Get and select vpls to scan
	foreach ($cms as $cmid => $cm) {
		$vpl = new mod_vpl($cmid);
		if ($vpl->is_visible() && $vpl->has_capability(VPL_VIEW_CAPABILITY)) {
			$vplid = $vpl->get_instance()->id;
			$vpls[$vplid]=$vpl;
			$grader = $grader || $vpl->has_capability(VPL_GRADE_CAPABILITY);
		}
	}
    if(count($vpls) ==0){ //No VPL to scan
    	return false;
    }
    $select = '(vpl IN ('.implode(',',array_keys($vpls)).'))';
    $select .= ' and ((datesubmitted >= '.$timestart.') or (dategraded >= '.$timestart.'))';
    if(!$grader){ //Own activity
    	$select = '(userid = '.$USER->id.') and '.$select;
    }
    $subs = get_records_select(VPL_SUBMISSIONS,$select,'datesubmitted DESC');
    if (!$subs){ //No submission available
        return false;
    }
    $activities = array();
    $norepeat = array();
    $norepeatuser = array();
    foreach ($subs as $sub) {
        //Show if own user or VPL_GRADE_CAPABILITY
        if(isset($norepeat[$sub->vpl]) || isset($norepeatuser[$sub->userid])){ //No repeat activity
        	continue;
        }
    	$vpl = $vpls[$sub->vpl];
        if(($USER->id == $sub->userid && $sub->datesubmitted >= $timestart) || $vpl->has_capability(VPL_GRADE_CAPABILITY) ){
        	$activities[] = $sub;
        }
        if(!$vpl->has_capability(VPL_GRADE_CAPABILITY)){ //No repeat activity if no grader capability
        	$norepeat[$sub->vpl] = true;
        }else{
        	$norepeatuser[$sub->userid] = 1;
        }
    }
    if(count($activities)){
    	print_headline(get_string('modulenameplural', VPL).':');
    	foreach ($activities as $sub) { //Show recent activity
    		$vpl = $vpls[$sub->vpl];
           	$url=vpl_mod_href('forms/submissionview.php','id',$vpl->get_course_module()->id,
           						'userid',$sub->userid,'submissionid',$sub->id);
           	if($sub->datesubmitted > $sub->dategraded || !$vpl->has_capability(VPL_GRADE_CAPABILITY)){
           		$str = $vpl->get_instance()->name.' ('.get_string('submission',VPL).')';
           		$date = $sub->datesubmitted;
           	}else{
           		$str = $vpl->get_instance()->name.' ('.get_string('grade').')';
           		$date = $sub->dategraded;
           	}
           	$user = get_record('user','id',$sub->userid);
    		print_recent_activity_note($date, $user, $str, $url, false, $grader);
    	}
    	return true;
    }
    return false;
}

/**
 * Run periodically to check for vpl visibility update
 *
 * @uses $CFG
 * @return boolean
 **/
function vpl_cron() {
	$now = time();
	$vpls = get_records_select(VPL,'','','id,visiblefrom,availablefrom,duedate');
	foreach ($vpls as $instance) {
		if($instance->visiblefrom==0){ // no set visiblefrom
			$start = $instance->availablefrom;
		}else if($instance->availablefrom==0){// no set availablefrom
			$start = $instance->visiblefrom;
		}else{
			$start = min($instance->visiblefrom,$instance->availablefrom);
		}
		if($start == 0){ //No start time
			continue;
		}
		if($instance->duedate>0 && $instance->duedate < $now){
			continue;  //After due time, no activate
		}
		if($start < $now && ($start+(2*3600)) >= $now){ //Not 2 hours late to set visible
			$vpl = new mod_vpl(null,$instance->id);
			if(!instance_is_visible(VPL,$vpl->get_instance())){
				echo 'Setting visible "'.s($vpl->get_instance()->name).'"';
				$cm = $vpl->get_course_module();
				set_coursemodule_visible($cm->id,true);
				rebuild_course_cache($cm->course);
			}
		}
	}
	return true;
}

/**
 * Must return an array of user records (all data) who are participants
 * for a given instance of vpl. Must include every user involved
 * in the instance, independient of his role (student, teacher, admin...)
 * See other modules as example.
 *
 * @param int $vplid ID of an instance of this module
 * @return mixed boolean/array of users
 **/
function vpl_get_participants($vplid) {
	global $CFG;
	//Locate students
	$submiters = get_records_sql("SELECT DISTINCT userid
	FROM {$CFG->prefix}vpl_submissions
	WHERE vpl = $vplid");
	//Locate graders
	$graders = get_records_sql("SELECT DISTINCT grader
	FROM {$CFG->prefix}vpl_submissions
	WHERE vpl = $vplid");

	//Read users records
	$participants = array();
	if($submiters){
		foreach ($submiters as $submiter) {
			$user = get_record('user','id',$submiter->userid);
			if($user){
				$participants[$user->id] = $user;
			}
		}
	}
	if($graders){
		foreach ($graders as $grader) {
			$user = get_record('user','id',$grader->grader);
			if($user && $grader->grader>0){ //Exist and Not automatic grader
				$participants[$user->id] = $user;
			}
		}
	}
	if(count($participants)>0){
		return $participants;
	}else{
		return false;
	}
}

function vpl_scale_used ($vplid,$scaleid) {
	return $scaleid and record_exists(VPL, 'id',"$vplid",'grade', "-$scaleid");
}


/**
 * Checks if scale is being used by any instance of vpl.
 * This is used to find out if scale used anywhere
 * @param $scaleid int
 * @return boolean True if the scale is used by any vpl
 */
function vpl_scale_used_anywhere($scaleid) {
	return $scaleid and record_exists(VPL, 'grade', "-$scaleid");
}

/**
 * Execute post-install custom actions for the module
 * This function was added in 1.9
 *
 * @return boolean true if success, false on error
 */
function vpl_install() {
	return true;
}

/**
 * Execute post-uninstall custom actions for the module
 * This function was added in 1.9
 *
 * @return boolean true if success, false on error
 */
function vpl_uninstall() {
	return true;
}

function vpl_get_view_actions() {
    return array('view', 'view all', 'view all submissions',  'run','debug', 'edit submission',
    			 'execution keep file form', 'execution limits form',
    			'edit full description', 'view grade', 'Diff', 'view similarity',
    			'view watermarks', 'similarity form', 'view previous'
    			);
}

function vpl_get_post_actions() {
    return array('save submision', 'evaluate', 'execution save keeplist',
    			'execution save limits', 'execution save options', 'execution options form',
    			'save full description', 'remove grade', 'upload submission', 'variations form'
    			);
}

/**
 * Removes all grades from gradebook
 * @param int $courseid
 * @param string optional type
 */
function vpl_reset_gradebook($courseid, $type='') {
    global $CFG;
	require_once $CFG->libdir.'/gradelib.php';
    if ($cms = get_coursemodules_in_course(VPL, $course->id)) {
		foreach ($cms as $cmid => $cm) {
			$vpl = new mod_vpl($cmid);
			$instance = $vpl->get_instance();
			$itemdetails = array('reset'=>1);
			grade_update('mod/vpl', $instance->course, 'mod', VPL,$instance->id, 0, NULL, $itemdetails);
		}
	}
}

/**
 * This function is used by the reset_course_userdata function in moodlelib.
 * This function will remove all posts from the specified vpl instance
 * and clean up any related data.
 * @param $data the data submitted from the reset course.
 * @return array status array
 */
function vpl_reset_userdata($data) {
	global $CFG;
    $status = array();
    if($data->reset_vpl_submissions){
    	$componentstr = get_string('modulenameplural', VPL);
		if ($cms = get_coursemodules_in_course(VPL, $data->courseid)) {
			foreach ($cms as $cmid => $cm) { //For each vpl instance in course
				$vpl = new mod_vpl($cmid);
				$instance = $vpl->get_instance();
				//Delete submissions records
				delete_records(VPL_SUBMISSIONS,'vpl',$instance->id);
				//Delete variations assigned
				delete_records(VPL_ASSIGNED_VARIATIONS,'vpl',$instance->id);
				//Delete submission files
				fulldelete(	$CFG->dataroot.'/vpl_data/'.$data->courseid.'/'.$instance->id.'/usersdata');
				$status[] = array('component'=>$componentstr, 'item'=>get_string('resetvpl', VPL, $instance->name), 'error'=>false);
			}
		}
    }
    return $status;
}

/**
 * Implementation of the function for printing the form elements that control
 * whether the course reset functionality affects the assignment.
 * @param $mform form passed by reference
 */
function vpl_reset_course_form_definition(&$mform) {
    $mform->addElement('header', 'vplheader', get_string('modulenameplural', VPL));
    $mform->addElement('advcheckbox', 'reset_vpl_submissions', get_string('deleteallsubmissions',VPL));
}

/**
 * Course reset form defaults.
 */
function vpl_reset_course_form_defaults($course) {
    return array('reset_vpl_submissions'=>1);
}

/*
 * any/all functions defined by the module should be in here. If the modulename is called widget, then the required functions include:
 * Other functions available but not required are:
 o widget_delete_course() - code to clean up anything that would be leftover after all instances are deleted
 o widget_process_options() - code to pre-process the form data from module settings
 o widget_reset_course_form() and widget_delete_userdata() - used to implement Reset course feature.
 * To avoid possible conflict, any module functions should be named starting with widget_ and any constants you define should start with WIDGET_
 */

?>