<?php
/**
 * Unit tests for mod/vpl/locallib.php.
 *
 * @version		$Id: testvpldiff.php,v 1.1 2010/05/26 16:14:20 juanca Exp $
 * @package		vpl. Unit tests for mod/vpl/similarity/diff.class.php
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino jcrodriguez@dis.ulpgc.es
 **/

if (!defined('MOODLE_INTERNAL')) {
	die('Direct access to this script is forbidden.');
}

require_once($CFG->dirroot.'/mod/vpl/similarity/diff.class.php');

class vpl_diff_test extends UnitTestCase {

	function test_removeAlphaNum(){
		$this->assertEqual(vpl_diff::removeAlphaNum(''),'');
		$this->assertEqual(vpl_diff::removeAlphaNum('aabcjzABGHWZ'),'');
		$this->assertEqual(vpl_diff::removeAlphaNum('0987654321'),'');
		$this->assertEqual(vpl_diff::removeAlphaNum('/,.:;-_'),'/,.:;-_');
		$this->assertEqual(vpl_diff::removeAlphaNum('a/0,b.B:;4-aZZ_'),'/,.:;-_');
	}

	function test_diffLine(){
		$this->assertEqual(vpl_diff::diffLine(" \t \t A=B*5"," \t \t A=B*5"),3);
		$this->assertEqual(vpl_diff::diffLine("A=B*5"," \t \t A=B*5"),3);
		$this->assertEqual(vpl_diff::diffLine("=*"," \t \t A=B*5"),2);
		$this->assertEqual(vpl_diff::diffLine(" \t \t"," \t"),1);
		$this->assertEqual(vpl_diff::diffLine("begin=*"," \t \t begin A=B*5"),2);
		$this->assertEqual(vpl_diff::diffLine(" \t \t"," \t \t begin A=B*5"),0);
		$this->assertEqual(vpl_diff::diffLine("begin","end"),0);
		$this->assertEqual(vpl_diff::diffLine("=*","*="),0);
		$this->assertEqual(vpl_diff::diffLine("L.Actual := 0;","lis.act := 1; --  principio"),2);
	}

	function test_initAuxiliarMatrices(){
		$nr=5;
		$nc=5;
		vpl_diff::initAuxiliarMatrices($matrix,$prev,$nr,$nc);
		for($i=1; $i<=$nr; $i++){
			for($j=1; $j<=$nc; $j++){
				$this->assertEqual($matrix[$i][$j],0);
				$this->assertEqual($prev[$i][$j],0);
			}
		}
		for($i=0; $i<=$nr; $i++){
			$this->assertEqual($matrix[$i][0],0);
			$this->assertEqual($prev[$i][0],-1);
		}
		for($j=1; $j<=$nc; $j++){
			$this->assertEqual($matrix[0][$j],0);
			$this->assertEqual($prev[0][$j],1);
		}
		$nr=2;
		$nc=3;
		vpl_diff::initAuxiliarMatrices($matrix,$prev,$nr,$nc);
		for($i=1; $i<=$nr; $i++){
			for($j=1; $j<=$nc; $j++){
				$this->assertEqual($matrix[$i][$j],0);
				$this->assertEqual($prev[$i][$j],0);
			}
		}
		for($i=0; $i<=$nr; $i++){
			$this->assertEqual($matrix[$i][0],0);
			$this->assertEqual($prev[$i][0],-1);
		}
		for($j=1; $j<=$nc; $j++){
			$this->assertEqual($matrix[0][$j],0);
			$this->assertEqual($prev[0][$j],1);
		}
	}
	function test_calculateDiff(){
		$ret=vpl_diff::calculateDiff(array(),array());
		$this->assertEqual($ret, false);
		$ret=vpl_diff::calculateDiff(array('begin'),array('begin'));
		if($this->assertEqual(count($ret), 1)){
			$this->assertEqual($ret[0]->type, '=');
			$this->assertEqual($ret[0]->ln1, 1);
			$this->assertEqual($ret[0]->ln2, 1);
		}
		$ret=vpl_diff::calculateDiff(array('begin'),array('end'));
		if($this->assertEqual(count($ret), 1)){
			$this->assertEqual($ret[0]->type, '#');
			$this->assertEqual($ret[0]->ln1, 1);
			$this->assertEqual($ret[0]->ln2, 1);
		}
		$ret=vpl_diff::calculateDiff(array('begin','end'),array('begin','end'));
		if($this->assertEqual(count($ret), 2)){
			for($i = 0; $i <count($ret); $i++){
				$this->assertEqual($ret[$i]->type, '=');
				$this->assertEqual($ret[$i]->ln1, $i+1);
				$this->assertEqual($ret[$i]->ln2, $i+1);
			}
		}
		$lines = array('begin','end','{','a=b*5+8 /*comment*/','}');
		$ret=vpl_diff::calculateDiff($lines, $lines);
		if($this->assertEqual(count($ret), count($lines))){
			for($i = 0; $i <count($ret); $i++){
				$this->assertEqual($ret[$i]->type, '=');
				$this->assertEqual($ret[$i]->ln1, $i+1);
				$this->assertEqual($ret[$i]->ln2, $i+1);
			}
		}
		$lines = array('begin','end','{','a=b*5+8 /*comment*/','}');
		$ret=vpl_diff::calculateDiff($lines, array());
		if($this->assertEqual(count($ret), count($lines))){
			for($i = 0; $i <count($ret); $i++){
				$this->assertEqual($ret[$i]->type, '<');
				$this->assertEqual($ret[$i]->ln1, $i+1);
			}
		}
		$lines = array('begin','end','{','a=b*5+8 /*comment*/','}');
		$ret=vpl_diff::calculateDiff(array(), $lines);
		if($this->assertEqual(count($ret), count($lines))){
			for($i = 0; $i <count($ret); $i++){
				$this->assertEqual($ret[$i]->type, '>');
				$this->assertEqual($ret[$i]->ln2, $i+1);
			}
		}
		$lines = array('begin','end','{','a=b*5+8 /*comment*/','}');
		$ret=vpl_diff::calculateDiff(array_merge($lines,$lines), $lines);
		if($this->assertEqual(count($ret), 2*count($lines))){
			for($i = 0; $i <count($lines); $i++){
				$this->assertEqual($ret[$i]->type, '<');
				$this->assertEqual($ret[$i]->ln1, $i+1);
			}
			for($i = count($lines); $i <2*count($lines); $i++){
				$this->assertEqual($ret[$i]->type, '=');
				$this->assertEqual($ret[$i]->ln1, $i+1);
				$this->assertEqual($ret[$i]->ln2, $i+1-count($lines));
			}
		}
		$lines = array('begin','end','{','a=b*5+8 /*comment*/','}');
		$ret=vpl_diff::calculateDiff($lines,array_merge($lines,$lines));
		if($this->assertEqual(count($ret), 2*count($lines))){
			for($i = 0; $i <count($lines); $i++){
				$this->assertEqual($ret[$i]->type, '>');
				$this->assertEqual($ret[$i]->ln2, $i+1);
			}
			for($i = count($lines); $i <2*count($lines); $i++){
				$this->assertEqual($ret[$i]->type, '=');
				$this->assertEqual($ret[$i]->ln1, $i+1-count($lines));
				$this->assertEqual($ret[$i]->ln2, $i+1);
			}
		}
		$lines = array('begin','end','{','a=b*5+8 /*comment*/','}');
		$ret=vpl_diff::calculateDiff($lines,array_merge($lines,$lines));
		if($this->assertEqual(count($ret), 2*count($lines))){
			for($i = 0; $i <count($lines); $i++){
				$this->assertEqual($ret[$i]->type, '>');
				$this->assertEqual($ret[$i]->ln2, $i+1);
			}
			for($i = count($lines); $i <2*count($lines); $i++){
				$this->assertEqual($ret[$i]->type, '=');
				$this->assertEqual($ret[$i]->ln1, $i+1-count($lines));
				$this->assertEqual($ret[$i]->ln2, $i+1);
			}
		}
	}
}

?>
